@@ -14,7 +14,7 @@ class Example(Generic[T]):
     table: T
 
 
-def load(data_dir: Union[str, Path], table_deser: Callable[[dict], T]) -> List[Example]:
+def load(data_dir: Union[str, Path], table_deser: Callable[[dict], T]) -> List[Example[T]]:
     """Load dataset from a folder. Assuming the following structure:
     - descriptions: (containing semantic descriptions of tables)
         - <table_fs_id>
